@@ -3,8 +3,10 @@
 # Computation tools to manipulate atmospheric variables.
 
 import numpy as np
+import xarray as xr
 from .utils_geo import get_south, get_north, apply_mask_axis
 
+
 ### ==================================== ###
 ###         Derivated variables          ###
 ### ==================================== ###
@@ -30,10 +32,11 @@ def omega2w(omega, p, T):
     omega, p, T = np.array(omega), np.array(p), np.array(T)
     R_gas = 287.058
     g = 9.80665
-    rho = p/(R_gas * T)
-    w = - omega / (rho*g)
+    rho = p / (R_gas * T)
+    w = -omega / (rho * g)
     return w
 
+
 # -- TODO -- Handle anyD velocity field.
 
 
@@ -61,26 +64,211 @@ def compute_vort(u, v, lat, lon):
 
     dlon = lon[1] - lon[0]  # resolution in longitude in deg
     dlat = lat[1] - lat[0]  # resolution in latitude in deg
-    R = 6371000            # Earth radius
+    R = 6371000  # Earth radius
     dy = R * dlat * np.pi / 180  # Vertical size of a cell
     # Horizontal size of a cell depending on latitude
-    lat_rad = lat * np.pi/180  # Latitudes in rad
-    r = np.sin(np.pi/2 - abs(lat_rad)) * R
+    lat_rad = lat * np.pi / 180  # Latitudes in rad
+    r = np.sin(np.pi / 2 - abs(lat_rad)) * R
     dx = r * dlon * np.pi / 180
-    dx = np.transpose([(dx[1:] + dx[:-1])/2] * (len(lon) - 1))
+    dx = np.transpose([(dx[1:] + dx[:-1]) / 2] * (len(lon) - 1))
 
     # Compute lat and lon corresponding to the vort matrix
     lat_vort = (lat[:-1] + lat[1:]) / 2
     lon_vort = (lon[:-1] + lon[1:]) / 2
 
-    W = np.zeros([len(lat)-1, len(lon)-1])  # Initialization of the matrix
-    W = (v[:-1, 1:]-v[:-1, :-1]) * 1/dx[:] - (u[1:, :-1] - u[:-1, :-1]) * 1/dy
+    W = np.zeros([len(lat) - 1, len(lon) - 1])  # Initialization of the matrix
+    W = (v[:-1, 1:] - v[:-1, :-1]) * 1 / dx[:] - (u[1:, :-1] - u[:-1, :-1]) * 1 / dy
 
     return lon_vort, lat_vort, W
 
 
+def get_dx_dy(lon, lat):
+    dlon = lon[1] - lon[0]  # resolution in longitude in deg
+    dlat = lat[1] - lat[0]  # resolution in latitude in deg
+    R = 6371000  # Earth radius
+    dy = R * dlat * np.pi / 180  # Vertical size of a cell
+    # Horizontal size of a cell depending on latitude
+    lat_rad = lat * np.pi / 180  # Latitudes in rad
+    r = np.sin(np.pi / 2 - abs(lat_rad)) * R
+    dx = r * dlon * np.pi / 180
+    dx = np.transpose([(dx[1:] + dx[:-1]) / 2] * (len(lon) - 1))
+    return dx, dy
+
+
+def compute_stretching_xr(
+    xarray, u_name="u", v_name="v", lon_name="lon", lat_name="lat"
+):
+    """Compute stretching deformation (E) from the horizontal velocity fields
+
+    Parameters
+    ----------
+    xarray : xr.Dataset
+        Dataset containing zonal and meridional wind in m/s
+    u_name, v_name, lon_name, lat_name : str
+        Names of variables in xarray, respectively zonal wind, meridional wind, longitude and latitude
+
+    Returns
+    -------
+    E : xr.Dataset
+        Stretching deformation in s-1
+    """
+
+    xarray = xarray.rename(
+        {lat_name: "lat", lon_name: "lon", u_name: "u", v_name: "v"}
+    ).squeeze()
+
+    lon, lat = xarray.lon.values, xarray.lat.values
+    dx, dy = get_dx_dy(xarray.lon.values, xarray.lat.values)
+
+    u = xarray.u
+    v = xarray.v
+    E = (
+        u.sel(lat=lat[:-1], lon=lon[1:]).values
+        - u.sel(lat=lat[:-1], lon=lon[:-1]).values
+    ) * 1 / dx[:] - (
+        v.sel(lat=lat[1:], lon=lon[:-1]).values - u.sel(lat=lat[:-1], lon=lon[:-1])
+    ) * 1 / dy
+    E = xr.DataArray(E).interp_like(xarray, kwargs={"fill_value": "extrapolate"})
+    E.attrs["units"] = "s-1"
+    E = E.rename({"lat": lat_name, "lon": lon_name})
+    return E
+
+
+def compute_shearing_xr(xarray, u_name="u", v_name="v", lon_name="lon", lat_name="lat"):
+    """Compute shearing deformation (F) from the horizontal velocity fields
+
+    Parameters
+    ----------
+    xarray : xr.Dataset
+        Dataset containing zonal and meridional wind in m/s
+    u_name, v_name, lon_name, lat_name : str
+        Names of variables in xarray, respectively zonal wind, meridional wind, longitude and latitude
+
+    Returns
+    -------
+    F : xr.Dataset
+        Shearing deformation in s-1
+    """
+
+    xarray = xarray.rename(
+        {lat_name: "lat", lon_name: "lon", u_name: "u", v_name: "v"}
+    ).squeeze()
+
+    lon, lat = xarray.lon.values, xarray.lat.values
+    dx, dy = get_dx_dy(xarray.lon.values, xarray.lat.values)
+
+    u = xarray.u
+    v = xarray.v
+    F = (
+        v.sel(lat=lat[:-1], lon=lon[1:]).values
+        - u.sel(lat=lat[:-1], lon=lon[:-1]).values
+    ) * 1 / dx[:] - (
+        u.sel(lat=lat[1:], lon=lon[:-1]).values - u.sel(lat=lat[:-1], lon=lon[:-1])
+    ) * 1 / dy
+    F = xr.DataArray(F).interp_like(xarray, kwargs={"fill_value": "extrapolate"})
+    F.attrs["units"] = "s-1"
+    F = F.rename({"lat": lat_name, "lon": lon_name})
+    return F
+
+
+def compute_ObukoWeiss_norm_xr(vort, E, F):
+    """Compute the normalized Obuko-Weiss Parameter
+
+    Parameters
+    ----------
+    vort : xr.Dataset
+        Vorticity field
+    E : xr.Dataset
+        Stretching deformation field
+    F : xr.Dataset
+        Shearing deformation field
+
+    Returns
+    -------
+    xr.Dataset
+        Normalized Obuko_Weiss parameter field
+    """
+    OW_n = 1 - ((E * 3600) ** 2 + (F * 3600) ** 2) / ((vort * 3600) ** 2)
+    OW_n.attrs["units"] = "1"
+    return OW_n
+
+
+def compute_OWZ_xr(vort, E, F, lat_name="lat"):
+    """Compute the Obuko-Weiss-Zeta Parameter (See Tory et al. 2013)
+
+    Parameters
+    ----------
+    vort : xr.Dataset
+        Vorticity field
+    E : xr.Dataset
+        Stretching deformation field
+    F : xr.Dataset
+        Shearing deformation field
+
+    Returns
+    -------
+    xr.Dataset
+        OWZ field
+    """
+    OW_n = compute_ObukoWeiss_norm_xr(vort, E, F)
+    f = compute_Coriolis_param(OW_n[lat_name])
+    zeros = xr.zeros_like(OW_n)
+    OWZ = xr.ufuncs.maximum(OW_n, zeros) * xr.ufuncs.sign(f) * (vort + f)
+    OWZ.attrs["units"] = "s-1"
+    return OWZ
+
+
+def compute_OWZ_from_files(
+    u_file,
+    v_file,
+    vo_file=None,
+    OWZ_file=None,
+    u_name="u",
+    v_name="v",
+    vo_name="vo",
+    lon_name="longitude",
+    lat_name="latitude",
+):
+    """
+
+    Parameters
+    ----------
+    u_file, v_file : str
+        Paths to the respective files containing zonal and meridional wind field
+    vo_file : str
+        Path to the file containing the vorticity field. To be implemented : If None, compute vorticity from u and v.
+    OWZ_file :
+        Path to which the OWZ field will be written. If None, not saved.
+    u_name, v_name, vo_name, lon_name, lat_name : str
+        Names of zonal wind, meridional wind, vorticity, longitude, latitude respectively in the files.
+
+    Returns
+    -------
+    OWZ : xr.Dataset
+        OWZ field
+    """
+    if vo_file == None:
+        # TODO : Implémenter compute_vort_xr et remplir cette partie pour calculer vo à partir de u et v
+        pass
+    else:
+        vo = xr.open_dataset(vo_file).squeeze()
+    u = xr.open_dataset(u_file).squeeze()
+    v = xr.open_dataset(v_file).squeeze()
+    wind = xr.merge([u, v])
+    wind = wind.rename({u_name: "u", v_name: "v", lon_name: "lon", lat_name: "lat"})
+    vo = vo.rename({vo_name: "vo", lon_name: "lon", lat_name: "lat"})
+
+    E = compute_stretching_xr(wind)
+    F = compute_shearing_xr(wind)
+    OWZ = compute_OWZ_xr(vo, E, F)
+    OWZ = OWZ.rename({"lat": lat_name, "lon": lon_name})
+    if OWZ_file != None:
+        OWZ.to_netcdf(OWZ_file)
+    return OWZ
+
+
 def compute_stretching(u, v, lat, lon):
-    """Compute stretching deformation from the horizontal velocity fields
+    """Compute stretching deformation (E) from the horizontal velocity fields
 
     ! So far, handles only 2D fields.
 
@@ -101,22 +289,30 @@ def compute_stretching(u, v, lat, lon):
         longitude, latitude and stretching field
     """
 
+    if type(lon) != np.ndarray:
+        lon = lon.values
+    if type(lat) != np.ndarray:
+        lat = lat.values
     dlon = lon[1] - lon[0]  # resolution in longitude in deg
     dlat = lat[1] - lat[0]  # resolution in latitude in deg
-    R = 6371000            # Earth radius
+    R = 6371000  # Earth radius
     dy = R * dlat * np.pi / 180  # Vertical size of a cell
     # Horizontal size of a cell depending on latitude
-    lat_rad = lat * np.pi/180  # Latitudes in rad
-    r = np.sin(np.pi/2 - abs(lat_rad)) * R
+    lat_rad = lat * np.pi / 180  # Latitudes in rad
+    r = np.sin(np.pi / 2 - abs(lat_rad)) * R
     dx = r * dlon * np.pi / 180
-    dx = np.transpose([(dx[1:] + dx[:-1])/2] * (len(lon) - 1))
+    dx = np.transpose([(dx[1:] + dx[:-1]) / 2] * (len(lon) - 1))
 
     # Compute lat and lon corresponding to the E matrix
     lat_E = (lat[:-1] + lat[1:]) / 2
     lon_E = (lon[:-1] + lon[1:]) / 2
 
-    E = np.zeros([len(lat)-1, len(lon)-1])  # Initialization of the matrix
-    E = (u[:-1, 1:]-u[:-1, :-1]) * 1/dx[:] - (v[1:, :-1] - v[:-1, :-1]) * 1/dy
+    # E = np.zeros([len(lat) - 1, len(lon) - 1])  # Initialization of the matrix
+    if type(u) != np.ndarray:
+        u = u.values
+    if type(v) != np.ndarray:
+        v = v.values
+    E = (u[:-1, 1:] - u[:-1, :-1]) * 1 / dx[:] - (v[1:, :-1] - v[:-1, :-1]) * 1 / dy
 
     return lon_E, lat_E, E
 
@@ -145,20 +341,20 @@ def compute_shearing(u, v, lat, lon):
 
     dlon = lon[1] - lon[0]  # resolution in longitude in deg
     dlat = lat[1] - lat[0]  # resolution in latitude in deg
-    R = 6371000            # Earth radius
+    R = 6371000  # Earth radius
     dy = R * dlat * np.pi / 180  # Vertical size of a cell
     # Horizontal size of a cell depending on latitude
-    lat_rad = lat * np.pi/180  # Latitudes in rad
-    r = np.sin(np.pi/2 - abs(lat_rad)) * R
+    lat_rad = lat * np.pi / 180  # Latitudes in rad
+    r = np.sin(np.pi / 2 - abs(lat_rad)) * R
     dx = r * dlon * np.pi / 180
-    dx = np.transpose([(dx[1:] + dx[:-1])/2] * (len(lon) - 1))
+    dx = np.transpose([(dx[1:] + dx[:-1]) / 2] * (len(lon) - 1))
 
     # Compute lat and lon corresponding to the F matrix
     lat_F = (lat[:-1] + lat[1:]) / 2
     lon_F = (lon[:-1] + lon[1:]) / 2
 
-    F = np.zeros([len(lat)-1, len(lon)-1])  # Initialization of the matrix
-    F = (v[:-1, 1:]-v[:-1, :-1]) * 1/dx[:] + (u[1:, :-1] - u[:-1, :-1]) * 1/dy
+    F = np.zeros([len(lat) - 1, len(lon) - 1])  # Initialization of the matrix
+    F = (v[:-1, 1:] - v[:-1, :-1]) * 1 / dx[:] + (u[1:, :-1] - u[:-1, :-1]) * 1 / dy
 
     return lon_F, lat_F, F
 
@@ -226,8 +422,7 @@ def compute_OWZ(vort, E, F, lat):
     OW_n = compute_ObukoWeiss_norm(vort, E, F)
     f = compute_Coriolis_param(lat)
     shape = np.shape(vort)
-    f = np.transpose(
-        np.array(list(f) * shape[1]).reshape([shape[1], shape[0]]))
+    f = np.transpose(np.array(list(f) * shape[1]).reshape([shape[1], shape[0]]))
     return np.sign(f) * (vort + f) * np.maximum(OW_n, np.zeros(np.shape(OW_n)))
 
 
@@ -250,30 +445,30 @@ def compute_grad(T, lat, lon):
     """
     dlon = lon[1] - lon[0]  # resolution in longitude in deg
     dlat = lat[1] - lat[0]  # resolution in latitude in deg
-    R = 6371000            # Earth radius
+    R = 6371000  # Earth radius
     dy = R * dlat * np.pi / 180  #
 
-    Gx = np.zeros([len(lat)-1, len(lon)-1])
-    Gy = np.zeros([len(lat)-1, len(lon)-1])
-    for i in range(len(lon)-1):  # i index of longitude
-        for j in range(len(lat)-1):  # j index of latitude
+    Gx = np.zeros([len(lat) - 1, len(lon) - 1])
+    Gy = np.zeros([len(lat) - 1, len(lon) - 1])
+    for i in range(len(lon) - 1):  # i index of longitude
+        for j in range(len(lat) - 1):  # j index of latitude
             # Compute dx geometrically
-            lat_rad = lat[j]*np.pi/180  # Current latitude in rad
+            lat_rad = lat[j] * np.pi / 180  # Current latitude in rad
             # radius of the current longitude circle
-            r = np.sin(np.pi/2 - abs(lat_rad))*R
+            r = np.sin(np.pi / 2 - abs(lat_rad)) * R
             dx = r * dlon * np.pi / 180
 
-            Gx[j, i] = (T[j+1, i] - T[j, i])/dx
-            Gy[j, i] = (T[j, i+1] - T[j, i])/dy
+            Gx[j, i] = (T[j + 1, i] - T[j, i]) / dx
+            Gy[j, i] = (T[j, i + 1] - T[j, i]) / dy
 
-    lat_G = np.array([(lat[j+1] + lat[j])/2 for j in range(len(lat) - 1)])
-    lon_G = np.array([(lon[i+1] + lon[i])/2 for i in range(len(lon) - 1)])
+    lat_G = np.array([(lat[j + 1] + lat[j]) / 2 for j in range(len(lat) - 1)])
+    lon_G = np.array([(lon[i + 1] + lon[i]) / 2 for i in range(len(lon) - 1)])
 
     return lon_G, lat_G, np.array(Gx), np.array(Gy)
 
 
 def compute_EKE(u, v):  # Probably deserves optimization if useful later.
-    """Compute Eddy Kinetic Energy from u and v fields. 
+    """Compute Eddy Kinetic Energy from u and v fields.
 
     Parameters
     ----------
@@ -295,11 +490,14 @@ def compute_EKE(u, v):  # Probably deserves optimization if useful later.
     U_zonal_lon = np.transpose([np.transpose(U_zonal) for i in range(nlon)])
     V_zonal_lon = np.transpose([np.transpose(V_zonal) for i in range(nlon)])
 
-    EKE = 0.5 * (np.multiply(u-U_zonal_lon, u-U_zonal_lon) +
-                 np.multiply(v-V_zonal_lon, v-V_zonal_lon))
+    EKE = 0.5 * (
+        np.multiply(u - U_zonal_lon, u - U_zonal_lon)
+        + np.multiply(v - V_zonal_lon, v - V_zonal_lon)
+    )
 
     return EKE
 
+
 ### ========================================== ###
 ###         Geographical computations          ###
 ### ========================================== ###
@@ -341,7 +539,7 @@ def hemispheric_mean(var, lat, axis=-1, neg=False):
     else:
         sign = +1
 
-    hemispheric_mean = var_north + sign*var_south
+    hemispheric_mean = var_north + sign * var_south
 
     return common_lat, hemispheric_mean
 
